@@ -86,7 +86,7 @@ static FILE *logfp = NULL;
 static int debug = (LOG_LEVEL_FATAL | LOG_LEVEL_ERROR);
 
 /* static functions */
-static void fatal_error(const char * error_message);
+static void fatal_error(const char *error_message);
 #ifdef _WIN32
 static char *w32_socket_strerr(int errcode, char *tmp_buf);
 #endif
@@ -339,7 +339,7 @@ void init_error_log(const char *prog_name, const char *logfname)
        * reopen it if the file name changed or if the
        * configuration reload was caused by a SIGHUP.
        */
-      log_error(LOG_LEVEL_INFO, "Failed to reopen logfile: \'%s\'. "
+      log_error(LOG_LEVEL_INFO, "Failed to reopen logfile: \'%s\': %E. "
          "Retrying after closing the old file descriptor first. If that "
          "doesn't work, Privoxy will exit without being able to log a message.",
          logfname);
@@ -352,7 +352,8 @@ void init_error_log(const char *prog_name, const char *logfname)
 
    if (NULL == fp)
    {
-      log_error(LOG_LEVEL_FATAL, "init_error_log(): can't open logfile: \'%s\'", logfname);
+      log_error(LOG_LEVEL_FATAL,
+         "init_error_log(): can't open logfile \'%s\': %E", logfname);
    }
 
 #ifdef FEATURE_EXTERNAL_FILTERS
@@ -583,7 +584,7 @@ static const char *get_log_level_string(int loglevel)
       case LOG_LEVEL_CONNECT:
          log_level_string = "Connect";
          break;
-     case LOG_LEVEL_TAGGING:
+      case LOG_LEVEL_TAGGING:
          log_level_string = "Tagging";
          break;
       case LOG_LEVEL_WRITING:
@@ -652,18 +653,18 @@ void log_error(int loglevel, const char *fmt, ...)
    char outbuf[LOG_BUFFER_SIZE+1];
    char tempbuf[LOG_BUFFER_SIZE];
    size_t length = 0;
-   const char * src = fmt;
+   const char *src = fmt;
    long thread_id;
    char timestamp[30];
    const size_t log_buffer_size = LOG_BUFFER_SIZE;
 
 #if defined(_WIN32) && !defined(_WIN_CONSOLE)
    /*
-    * Irrespective of debug setting, a GET/POST/CONNECT makes
-    * the taskbar icon animate.  (There is an option to disable
-    * this but checking that is handled inside LogShowActivity()).
+    * Irrespective of debug setting, a request makes the taskbar icon
+    * animate. (There is an option to disable this but checking that is
+    * handled inside LogShowActivity()).
     */
-   if ((loglevel == LOG_LEVEL_REQUEST) || (loglevel == LOG_LEVEL_CRUNCH))
+   if (loglevel == LOG_LEVEL_REQUEST)
    {
       LogShowActivity();
    }
@@ -692,8 +693,8 @@ void log_error(int loglevel, const char *fmt, ...)
 #endif
       if (loglevel == LOG_LEVEL_FATAL)
       {
-         fatal_error("Fatal error. You're not supposed to"
-            "see this message. Please file a bug report.");
+         fatal_error("Fatal error. You're not supposed to "
+            "see this message. Please file a bug report.\n");
       }
       return;
    }
