@@ -7,7 +7,7 @@
  *                using mbedTLS.
  *
  * Copyright   :  Written by and Copyright (c) 2017-2020 Vaclav Svec. FIT CVUT.
- *                Copyright (C) 2018-2021 by Fabian Keil <fk@fabiankeil.de>
+ *                Copyright (C) 2018-2024 by Fabian Keil <fk@fabiankeil.de>
  *
  *                This program is free software; you can redistribute it
  *                and/or modify it under the terms of the GNU General
@@ -32,13 +32,8 @@
 #include <string.h>
 #include <unistd.h>
 
-#if !defined(MBEDTLS_CONFIG_FILE)
-#  include "mbedtls/config.h"
-#else
-#  include MBEDTLS_CONFIG_FILE
-#endif
-
-#include "mbedtls/md5.h"
+#include <mbedtls/version.h>
+#include "mbedtls/sha256.h"
 #include "mbedtls/pem.h"
 #include "mbedtls/base64.h"
 #include "mbedtls/error.h"
@@ -143,59 +138,37 @@ extern size_t is_ssl_pending(struct ssl_attr *ssl_attr)
 extern int ssl_send_data(struct ssl_attr *ssl_attr, const unsigned char *buf, size_t len)
 {
    mbedtls_ssl_context *ssl = &ssl_attr->mbedtls_attr.ssl;
-   int ret = 0;
-   size_t max_fragment_size = 0;  /* Maximal length of data in one SSL fragment*/
-   int send_len             = 0;  /* length of one data part to send */
-   int pos                  = 0;  /* Position of unsent part in buffer */
+   int pos = 0; /* Position of unsent part in buffer */
 
    if (len == 0)
    {
       return 0;
    }
 
-   /* Getting maximal length of data sent in one fragment */
-   max_fragment_size = mbedtls_ssl_get_max_frag_len(ssl);
-
-   /*
-    * Whole buffer must be sent in many fragments, because each fragment
-    * has its maximal length.
-    */
    while (pos < len)
    {
-      /* Compute length of data, that can be send in next fragment */
-      if ((pos + (int)max_fragment_size) > len)
-      {
-         send_len = (int)len - pos;
-      }
-      else
-      {
-         send_len = (int)max_fragment_size;
-      }
+      int ret;
+      int send_len;
+
+      send_len = (int)len - pos;
 
       log_error(LOG_LEVEL_WRITING, "TLS on socket %d: %N",
          ssl_attr->mbedtls_attr.socket_fd.fd, send_len, buf+pos);
 
-      /*
-       * Sending one part of the buffer
-       */
-      while ((ret = mbedtls_ssl_write(ssl,
-         (const unsigned char *)(buf + pos),
-         (size_t)send_len)) < 0)
+      ret = mbedtls_ssl_write(ssl, (const unsigned char *)(buf + pos),
+         (size_t)send_len);
+      if (ret <= 0)
       {
-         if (ret != MBEDTLS_ERR_SSL_WANT_READ &&
-             ret != MBEDTLS_ERR_SSL_WANT_WRITE)
-         {
-            char err_buf[ERROR_BUF_SIZE];
+         char err_buf[ERROR_BUF_SIZE];
 
-            mbedtls_strerror(ret, err_buf, sizeof(err_buf));
-            log_error(LOG_LEVEL_ERROR,
-               "Sending data on socket %d over TLS/SSL failed: %s",
-               ssl_attr->mbedtls_attr.socket_fd.fd, err_buf);
-            return -1;
-         }
+         mbedtls_strerror(ret, err_buf, sizeof(err_buf));
+         log_error(LOG_LEVEL_ERROR,
+            "Sending %d bytes on socket %d over TLS/SSL failed with ret %d: %s",
+            send_len, ssl_attr->mbedtls_attr.socket_fd.fd, ret, err_buf);
+         return -1;
       }
-      /* Adding count of sent bytes to position in buffer */
-      pos = pos + send_len;
+
+      pos = pos + ret;
    }
 
    return (int)len;
@@ -240,7 +213,7 @@ extern int ssl_recv_data(struct ssl_attr *ssl_attr, unsigned char *buf, size_t m
       if (ret == MBEDTLS_ERR_SSL_PEER_CLOSE_NOTIFY)
       {
          log_error(LOG_LEVEL_CONNECT, "The peer notified us that "
-            "the connection on socket %d is going to be closed",
+            "the connection on socket %d is going to be closed.",
             ssl_attr->mbedtls_attr.socket_fd.fd);
          return 0;
       }
@@ -325,17 +298,16 @@ extern int create_client_ssl_connection(struct client_state *csp)
     * certificate and key inconsistence must be locked.
     */
    privoxy_mutex_lock(&certificate_mutex);
-
    ret = generate_host_certificate(csp);
+   privoxy_mutex_unlock(&certificate_mutex);
+
    if (ret < 0)
    {
       log_error(LOG_LEVEL_ERROR,
          "generate_host_certificate failed: %d", ret);
-      privoxy_mutex_unlock(&certificate_mutex);
       ret = -1;
       goto exit;
    }
-   privoxy_mutex_unlock(&certificate_mutex);
 
    /*
     * Seed the RNG
@@ -372,8 +344,13 @@ extern int create_client_ssl_connection(struct client_state *csp)
       goto exit;
    }
 
+#if MBEDTLS_VERSION_MAJOR < 3
    ret = mbedtls_pk_parse_keyfile(&(ssl_attr->mbedtls_attr.prim_key),
       key_file, NULL);
+#else
+   ret = mbedtls_pk_parse_keyfile(&(ssl_attr->mbedtls_attr.prim_key),
+      key_file, NULL, mbedtls_ctr_drbg_random, &ctr_drbg);
+#endif
    if (ret != 0)
    {
       mbedtls_strerror(ret, err_buf, sizeof(err_buf));
@@ -430,7 +407,7 @@ extern int create_client_ssl_connection(struct client_state *csp)
       if (ssl_attr->mbedtls_attr.ciphersuites_list == NULL)
       {
          log_error(LOG_LEVEL_ERROR,
-            "Setting the cipher list '%s' for the client connection failed",
+            "Setting the cipher list '%s' for the client connection failed.",
             csp->config->cipher_list);
          ret = -1;
          goto exit;
@@ -474,7 +451,7 @@ extern int create_client_ssl_connection(struct client_state *csp)
       {
          mbedtls_strerror(ret, err_buf, sizeof(err_buf));
          log_error(LOG_LEVEL_ERROR,
-            "medtls_ssl_handshake with client failed: %s", err_buf);
+            "mbedtls_ssl_handshake with client failed: %s", err_buf);
          ret = -1;
          goto exit;
       }
@@ -682,7 +659,7 @@ extern int create_server_ssl_connection(struct client_state *csp)
       if (ssl_attr->mbedtls_attr.ciphersuites_list == NULL)
       {
          log_error(LOG_LEVEL_ERROR,
-            "Setting the cipher list '%s' for the server connection failed",
+            "Setting the cipher list '%s' for the server connection failed.",
             csp->config->cipher_list);
          ret = -1;
          goto exit;
@@ -723,7 +700,7 @@ extern int create_server_ssl_connection(struct client_state *csp)
     * Handshake with server
     */
    log_error(LOG_LEVEL_CONNECT,
-      "Performing the TLS/SSL handshake with the server");
+      "Performing the TLS/SSL handshake with the server.");
 
    while ((ret = mbedtls_ssl_handshake(&(ssl_attr->mbedtls_attr.ssl))) != 0)
    {
@@ -897,7 +874,7 @@ static int write_certificate(mbedtls_x509write_cert *crt, const char *output_fil
     */
    if ((f = fopen(output_file, "w")) == NULL)
    {
-      log_error(LOG_LEVEL_ERROR, "Opening file %s to save certificate failed",
+      log_error(LOG_LEVEL_ERROR, "Opening file %s to save certificate failed.",
          output_file);
       return -1;
    }
@@ -905,7 +882,7 @@ static int write_certificate(mbedtls_x509write_cert *crt, const char *output_fil
    if (fwrite(cert_buf, 1, len, f) != len)
    {
       log_error(LOG_LEVEL_ERROR,
-         "Writing certificate into file %s failed", output_file);
+         "Writing certificate into file %s failed.", output_file);
       fclose(f);
       return -1;
    }
@@ -974,7 +951,7 @@ static int write_private_key(mbedtls_pk_context *key, unsigned char **ret_buf,
    {
       fclose(f);
       log_error(LOG_LEVEL_ERROR,
-         "Writing private key into file %s failed",
+         "Writing private key into file %s failed.",
          key_file_path);
       ret = -1;
       goto exit;
@@ -1088,7 +1065,7 @@ static int generate_key(struct client_state *csp, unsigned char **key_buf)
    if ((ret = write_private_key(&key, key_buf, key_opt.key_file_path)) < 0)
    {
       log_error(LOG_LEVEL_ERROR,
-         "Writing private key into file %s failed", key_opt.key_file_path);
+         "Writing private key into file %s failed.", key_opt.key_file_path);
       ret = -1;
       goto exit;
    }
@@ -1286,7 +1263,11 @@ static int generate_host_certificate(struct client_state *csp)
    mbedtls_pk_context *issuer_key  = &loaded_issuer_key;
    mbedtls_pk_context *subject_key = &loaded_subject_key;
    mbedtls_x509write_cert cert;
+#if MBEDTLS_VERSION_MAJOR < 3
    mbedtls_mpi serial;
+#else
+   unsigned char serial_buf[16];
+#endif
 
    unsigned char *key_buf = NULL;    /* Buffer for created key */
 
@@ -1371,7 +1352,7 @@ static int generate_host_certificate(struct client_state *csp)
    {
       freez(cert_opt.output_file);
       freez(cert_opt.subject_key);
-      log_error(LOG_LEVEL_ERROR, "Key generating failed");
+      log_error(LOG_LEVEL_ERROR, "Key generating failed.");
       return -1;
    }
 
@@ -1382,7 +1363,9 @@ static int generate_host_certificate(struct client_state *csp)
    mbedtls_x509write_crt_set_md_alg(&cert, CERT_SIGNATURE_ALGORITHM);
    mbedtls_pk_init(&loaded_issuer_key);
    mbedtls_pk_init(&loaded_subject_key);
+#if MBEDTLS_VERSION_MAJOR < 3
    mbedtls_mpi_init(&serial);
+#endif
    mbedtls_x509_crt_init(&issuer_cert);
 
    /*
@@ -1396,6 +1379,7 @@ static int generate_host_certificate(struct client_state *csp)
    char cert_params[cert_params_len];
    memset(cert_params, 0, cert_params_len);
 
+#if MBEDTLS_VERSION_MAJOR < 3
    /*
     * Converting unsigned long serial number to char * serial number.
     * We must compute length of serial number in string + terminating null.
@@ -1415,10 +1399,11 @@ static int generate_host_certificate(struct client_state *csp)
    if (ret < 0 || ret >= serial_num_size)
    {
       log_error(LOG_LEVEL_ERROR,
-         "Converting certificate serial number into string failed");
+         "Converting certificate serial number into string failed.");
       ret = -1;
       goto exit;
    }
+#endif
 
    /*
     * Preparing parameters for certificate
@@ -1437,7 +1422,7 @@ static int generate_host_certificate(struct client_state *csp)
    if (get_certificate_valid_from_date(cert_valid_from, sizeof(cert_valid_from), VALID_DATETIME_FMT)
     || get_certificate_valid_to_date(cert_valid_to, sizeof(cert_valid_to), VALID_DATETIME_FMT))
    {
-      log_error(LOG_LEVEL_ERROR, "Generating one of the validity dates failed");
+      log_error(LOG_LEVEL_ERROR, "Generating one of the validity dates failed.");
       ret = -1;
       goto exit;
    }
@@ -1447,7 +1432,9 @@ static int generate_host_certificate(struct client_state *csp)
    cert_opt.subject_name  = cert_params;
    cert_opt.not_before    = cert_valid_from;
    cert_opt.not_after     = cert_valid_to;
+#if MBEDTLS_VERSION_MAJOR < 3
    cert_opt.serial        = serial_num_text;
+#endif
    cert_opt.is_ca         = 0;
    cert_opt.max_pathlen   = -1;
 
@@ -1457,7 +1444,7 @@ static int generate_host_certificate(struct client_state *csp)
     */
    if (subject_key_len == 0)
    {
-      log_error(LOG_LEVEL_ERROR, "Subject key was already created");
+      log_error(LOG_LEVEL_ERROR, "Subject key was already created.");
       ret = 0;
       goto exit;
    }
@@ -1472,6 +1459,7 @@ static int generate_host_certificate(struct client_state *csp)
       goto exit;
    }
 
+#if MBEDTLS_VERSION_MAJOR < 3
    /*
     * Parse serial to MPI
     */
@@ -1484,6 +1472,7 @@ static int generate_host_certificate(struct client_state *csp)
       ret = -1;
       goto exit;
    }
+#endif
 
    /*
     * Loading certificates
@@ -1514,15 +1503,28 @@ static int generate_host_certificate(struct client_state *csp)
    if (key_buf != NULL && subject_key_len > 0)
    {
       /* Key was created in this function and is stored in buffer */
+#if MBEDTLS_VERSION_MAJOR < 3
       ret = mbedtls_pk_parse_key(&loaded_subject_key, key_buf,
          (size_t)(subject_key_len + 1), (unsigned const char *)
          cert_opt.subject_pwd, strlen(cert_opt.subject_pwd));
+#else
+      ret = mbedtls_pk_parse_key(&loaded_subject_key, key_buf,
+         (size_t)(subject_key_len + 1), (unsigned const char *)
+         cert_opt.subject_pwd, strlen(cert_opt.subject_pwd),
+         mbedtls_ctr_drbg_random, &ctr_drbg);
+#endif
    }
    else
    {
       /* Key wasn't created in this function, because it already existed */
+#if MBEDTLS_VERSION_MAJOR < 3
       ret = mbedtls_pk_parse_keyfile(&loaded_subject_key,
          cert_opt.subject_key, cert_opt.subject_pwd);
+#else
+      ret = mbedtls_pk_parse_keyfile(&loaded_subject_key,
+         cert_opt.subject_key, cert_opt.subject_pwd,
+         mbedtls_ctr_drbg_random, &ctr_drbg);
+#endif
    }
 
    if (ret != 0)
@@ -1534,8 +1536,13 @@ static int generate_host_certificate(struct client_state *csp)
       goto exit;
    }
 
+#if MBEDTLS_VERSION_MAJOR < 3
    ret = mbedtls_pk_parse_keyfile(&loaded_issuer_key, cert_opt.issuer_key,
       cert_opt.issuer_pwd);
+#else
+   ret = mbedtls_pk_parse_keyfile(&loaded_issuer_key, cert_opt.issuer_key,
+      cert_opt.issuer_pwd, mbedtls_ctr_drbg_random, &ctr_drbg);
+#endif
    if (ret != 0)
    {
       mbedtls_strerror(ret, err_buf, sizeof(err_buf));
@@ -1545,21 +1552,6 @@ static int generate_host_certificate(struct client_state *csp)
       goto exit;
    }
 
-   /*
-    * Check if key and issuer certificate match
-    */
-   if (!mbedtls_pk_can_do(&issuer_cert.pk, MBEDTLS_PK_RSA) ||
-      mbedtls_mpi_cmp_mpi(&mbedtls_pk_rsa(issuer_cert.pk)->N,
-         &mbedtls_pk_rsa(*issuer_key)->N) != 0 ||
-      mbedtls_mpi_cmp_mpi(&mbedtls_pk_rsa(issuer_cert.pk)->E,
-         &mbedtls_pk_rsa(*issuer_key)->E) != 0)
-   {
-      log_error(LOG_LEVEL_ERROR,
-         "Issuer key doesn't match issuer certificate");
-      ret = -1;
-      goto exit;
-   }
-
    mbedtls_x509write_crt_set_subject_key(&cert, subject_key);
    mbedtls_x509write_crt_set_issuer_key(&cert, issuer_key);
 
@@ -1586,7 +1578,13 @@ static int generate_host_certificate(struct client_state *csp)
       goto exit;
    }
 
+#if MBEDTLS_VERSION_MAJOR < 3
    ret = mbedtls_x509write_crt_set_serial(&cert, &serial);
+#else
+   mbedtls_ctr_drbg_random(&ctr_drbg, serial_buf, sizeof(serial_buf));
+   ret = mbedtls_x509write_crt_set_serial_raw(&cert,
+      (unsigned char *)&serial_buf, sizeof(serial_buf));
+#endif
    if (ret != 0)
    {
       mbedtls_strerror(ret, err_buf, sizeof(err_buf));
@@ -1660,7 +1658,7 @@ static int generate_host_certificate(struct client_state *csp)
       mbedtls_ctr_drbg_random, &ctr_drbg);
    if (ret < 0)
    {
-      log_error(LOG_LEVEL_ERROR, "Writing certificate into file failed");
+      log_error(LOG_LEVEL_ERROR, "Writing certificate into file failed.");
       goto exit;
    }
 
@@ -1671,7 +1669,9 @@ exit:
    mbedtls_x509write_crt_free(&cert);
    mbedtls_pk_free(&loaded_subject_key);
    mbedtls_pk_free(&loaded_issuer_key);
+#if MBEDTLS_VERSION_MAJOR < 3
    mbedtls_mpi_free(&serial);
+#endif
    mbedtls_x509_crt_free(&issuer_cert);
 
    freez(cert_opt.subject_key);
@@ -1730,7 +1730,7 @@ static int ssl_verify_callback(void *csp_void, mbedtls_x509_crt *crt,
    if (MBEDTLS_ERR_BASE64_BUFFER_TOO_SMALL != ret)
    {
       log_error(LOG_LEVEL_ERROR,
-         "Failed to figure out the required X509 PEM certificate buffer size");
+         "Failed to figure out the required X509 PEM certificate buffer size.");
       return -1;
    }
    pem_buffer_length = olen;
@@ -1739,7 +1739,7 @@ static int ssl_verify_callback(void *csp_void, mbedtls_x509_crt *crt,
    if (last->file_buf == NULL)
    {
       log_error(LOG_LEVEL_ERROR,
-         "Failed to allocate %lu bytes to store the X509 PEM certificate",
+         "Failed to allocate %lu bytes to store the X509 PEM certificate.",
          pem_buffer_length);
       return -1;
    }
@@ -1773,7 +1773,7 @@ static int ssl_verify_callback(void *csp_void, mbedtls_x509_crt *crt,
       if (encoded_text == NULL)
       {
          log_error(LOG_LEVEL_ERROR,
-            "Failed to HTML-encode the certificate information");
+            "Failed to HTML-encode the certificate information.");
          return -1;
       }
       strlcpy(last->info_buf, encoded_text, sizeof(last->info_buf));
@@ -1788,8 +1788,8 @@ static int ssl_verify_callback(void *csp_void, mbedtls_x509_crt *crt,
  *
  * Function    :  host_to_hash
  *
- * Description :  Creates MD5 hash from host name. Host name is loaded
- *                from structure csp and saved again into it.
+ * Description :  Creates a sha256 hash from host name. The host name
+ *                is taken from the csp structure and stored into it.
  *
  * Parameters  :
  *          1  :  csp = Current client state (buffers, headers, etc...)
@@ -1800,36 +1800,11 @@ static int ssl_verify_callback(void *csp_void, mbedtls_x509_crt *crt,
  *********************************************************************/
 static int host_to_hash(struct client_state *csp)
 {
-   int ret = 0;
+   mbedtls_sha256((unsigned char *)csp->http->host,
+      strlen(csp->http->host), csp->http->hash_of_host, 0);
 
-#if !defined(MBEDTLS_MD5_C)
-#error mbedTLS needs to be compiled with md5 support
-#else
-   memset(csp->http->hash_of_host, 0, sizeof(csp->http->hash_of_host));
-   ret = mbedtls_md5_ret((unsigned char *)csp->http->host,
-      strlen(csp->http->host), csp->http->hash_of_host);
-   if (ret != 0)
-   {
-      log_error(LOG_LEVEL_ERROR,
-         "Failed to generate md5 hash of host %s: %d",
-         csp->http->host, ret);
-      return -1;
-   }
+   return create_hexadecimal_hash_of_host(csp);
 
-   /* Converting hash into string with hex */
-   size_t i = 0;
-   for (; i < 16; i++)
-   {
-      if ((ret = sprintf((char *)csp->http->hash_of_host_hex + 2 * i, "%02x",
-         csp->http->hash_of_host[i])) < 0)
-      {
-         log_error(LOG_LEVEL_ERROR, "Sprintf return value: %d", ret);
-         return -1;
-      }
-   }
-
-   return 0;
-#endif /* MBEDTLS_MD5_C */
 }
 
 /*********************************************************************
@@ -1855,6 +1830,15 @@ static int seed_rng(struct client_state *csp)
       privoxy_mutex_lock(&ssl_init_mutex);
       if (rng_seeded == 0)
       {
+#if MBEDTLS_VERSION_MAJOR >= 3
+         psa_status_t status = psa_crypto_init();
+         if (PSA_SUCCESS != status)
+         {
+            log_error(LOG_LEVEL_ERROR, "psa_crypto_init failed: %d", status);
+            privoxy_mutex_unlock(&ssl_init_mutex);
+            return -1;
+         }
+#endif
          mbedtls_ctr_drbg_init(&ctr_drbg);
          mbedtls_entropy_init(&entropy);
          ret = mbedtls_ctr_drbg_seed(&ctr_drbg, mbedtls_entropy_func,
