@@ -297,7 +297,7 @@ strptime_internal (rp, fmt, tm, decided, era_cnt)
 	}
 
       /* Any character but `%' must be matched by the same character
-	 in the iput string.  */
+	 in the input string.  */
       if (*fmt != '%')
 	{
 	  match_char (*fmt++, *rp++);
